import sys

import sqids

from hypothesis import given, target, assume
import hypothesis.strategies as st


lists_of_integers = st.lists(
    elements=st.integers(min_value=0, max_value=sys.maxsize),
    min_size=1,
)
min_lengths = st.integers(min_value=0, max_value=255)
alphabets = st.text(
    alphabet=st.characters(min_codepoint=0, max_codepoint=0x7F),
    min_size=3,
)


@given(numbers=lists_of_integers, min_length=min_lengths, alphabet=alphabets)
def test_round_trip_encoding(numbers, min_length, alphabet):
    # Encourage hypothesis to find ideal alphabets
    # by giving unique alphabets a score of 1.0
    # and non-unique alphabets a lower score.
    target(len(set(alphabet)) / len(alphabet))
    # Reject non-unique alphabets without failing the test.
    assume(len(set(alphabet)) == len(alphabet))

    sqid_1 = sqids.Sqids(min_length=min_length, alphabet=alphabet, blocklist=[])
    id_1 = sqid_1.encode(numbers)
    assert sqid_1.decode(id_1) == numbers

    # If the ID is long enough, use it as a blocklist word and ensure it is blocked.
    if len(id_1) >= 3:  # pragma: nocover
        sqid_2 = sqids.Sqids(min_length=min_length, alphabet=alphabet, blocklist=[id_1])
        id_2 = sqid_2.encode(numbers)
        assert id_1 != id_2
        assert sqid_2.decode(id_2) == numbers



import pytest
import sys
from sqids import Sqids


def test_simple():
    sqids = Sqids()
    numbers = [1, 2, 3]
    id_str = "86Rf07"
    assert sqids.encode(numbers) == id_str
    assert sqids.decode(id_str) == numbers


def test_different_inputs():
    sqids = Sqids()
    numbers = [0, 0, 0, 1, 2, 3, 100, 1_000, 100_000, 1_000_000, sys.maxsize]
    assert sqids.decode(sqids.encode(numbers)) == numbers


@pytest.mark.parametrize(
    "id_str, numbers",
    (
        ("bM", [0]),
        ("Uk", [1]),
        ("gb", [2]),
        ("Ef", [3]),
        ("Vq", [4]),
        ("uw", [5]),
        ("OI", [6]),
        ("AX", [7]),
        ("p6", [8]),
        ("nJ", [9]),
    )
)
def test_incremental_numbers(id_str, numbers):
    sqids = Sqids()
    assert sqids.encode(numbers) == id_str
    assert sqids.decode(id_str) == numbers


@pytest.mark.parametrize(
    "id_str, numbers",
    (
        ("SvIz", [0, 0]),
        ("n3qa", [0, 1]),
        ("tryF", [0, 2]),
        ("eg6q", [0, 3]),
        ("rSCF", [0, 4]),
        ("sR8x", [0, 5]),
        ("uY2M", [0, 6]),
        ("74dI", [0, 7]),
        ("30WX", [0, 8]),
        ("moxr", [0, 9]),
    )
)
def test_incremental_numbers_same_index_0(id_str, numbers):
    sqids = Sqids()
    assert sqids.encode(numbers) == id_str
    assert sqids.decode(id_str) == numbers


@pytest.mark.parametrize(
    "id_str, numbers",
    (
        ("SvIz", [0, 0]),
        ("nWqP", [1, 0]),
        ("tSyw", [2, 0]),
        ("eX68", [3, 0]),
        ("rxCY", [4, 0]),
        ("sV8a", [5, 0]),
        ("uf2K", [6, 0]),
        ("7Cdk", [7, 0]),
        ("3aWP", [8, 0]),
        ("m2xn", [9, 0]),
    )
)
def test_incremental_numbers_same_index_1(id_str, numbers):
    sqids = Sqids()
    assert sqids.encode(numbers) == id_str
    assert sqids.decode(id_str) == numbers


def test_multi_input():
    sqids = Sqids()
    numbers = list(range(100))
    output = sqids.decode(sqids.encode(numbers))
    assert numbers == output


def test_encoding_no_numbers():
    sqids = Sqids()
    assert sqids.encode([]) == ""


def test_decoding_empty_string():
    sqids = Sqids()
    assert sqids.decode("") == []


def test_decoding_invalid_character():
    sqids = Sqids()
    assert sqids.decode("*") == []


def test_encode_out_of_range_numbers():
    sqids = Sqids()
    with pytest.raises(ValueError):
        sqids.encode([-1])
    with pytest.raises(ValueError):
        sqids.encode([sys.maxsize + 1])



from typing import List, Set, Tuple
import sys
from .constants import (
    DEFAULT_ALPHABET,
    DEFAULT_BLOCKLIST,
    DEFAULT_MIN_LENGTH,
    _exact_match,
    _match_at_ends,
    _match_anywhere,
)

DIGITS = set("0123456789")


class Sqids:
    def __init__(
        self,
        alphabet: str = DEFAULT_ALPHABET,
        min_length: int = DEFAULT_MIN_LENGTH,
        blocklist: List[str] = DEFAULT_BLOCKLIST,
    ):
        if any(ord(char) > 127 for char in alphabet):
            raise ValueError("Alphabet cannot contain multibyte characters")

        if len(alphabet) < 3:
            raise ValueError("Alphabet length must be at least 3")

        if len(set(alphabet)) != len(alphabet):
            raise ValueError("Alphabet must contain unique characters")

        if not isinstance(min_length, int) or isinstance(min_length, bool):
            raise TypeError("Minimum length must be an int")

        MIN_LENGTH_LIMIT = 255
        if min_length < 0 or min_length > MIN_LENGTH_LIMIT:
            raise ValueError(
                f"Minimum length has to be between 0 and {MIN_LENGTH_LIMIT}"
            )

        # When the blocklist and alphabet are defaults, use pre-filtered blocklists.
        if blocklist is DEFAULT_BLOCKLIST and alphabet is DEFAULT_ALPHABET:
            self.__blocklist_exact_match: Set[str] = _exact_match
            self.__blocklist_match_at_ends: Tuple[str, ...] = _match_at_ends
            self.__blocklist_match_anywhere: Set[str] = _match_anywhere
        else:
            alphabet_lower = set(alphabet.lower())
            exact_match: Set[str] = set()
            match_at_ends: Set[str] = set()
            match_anywhere: Set[str] = set()
            for word in blocklist:
                if len(word) < 3:
                    continue
                word_lower = word.lower()
                word_lower_set = set(word_lower)
                if word_lower_set & alphabet_lower != word_lower_set:
                    continue

                if len(word) == 3:
                    exact_match.add(word.lower())
                elif word_lower_set & DIGITS:
                    match_at_ends.add(word_lower)
                else:
                    match_anywhere.add(word_lower)

            self.__blocklist_exact_match = exact_match
            # When matching at the ends, `.startswith()` and `.endswith()` need a tuple.
            self.__blocklist_match_at_ends = tuple(match_at_ends)
            self.__blocklist_match_anywhere = match_anywhere

        self.__alphabet = self.__shuffle(alphabet)
        self.__min_length = min_length

    def encode(self, numbers: List[int]) -> str:
        if not numbers:
            return ""

        if not all(0 <= number <= sys.maxsize for number in numbers):
            raise ValueError(f"Encoding supports numbers between 0 and {sys.maxsize}")

        return self.__encode_numbers(numbers, 0)

    def __encode_numbers(self, numbers: List[int], increment: int = 0) -> str:
        if increment > len(self.__alphabet):
            raise ValueError("Reached max attempts to re-generate the ID")

        offset = sum(
            (
                ord(self.__alphabet[v % len(self.__alphabet)]) + i
                for i, v in enumerate(numbers)
            )
        )
        offset = (offset + len(numbers)) % len(self.__alphabet)
        offset = (offset + increment) % len(self.__alphabet)
        alphabet = self.__alphabet[offset:] + self.__alphabet[:offset]
        prefix = alphabet[0]
        alphabet = alphabet[::-1]

        ret = [prefix]

        for i, num in enumerate(numbers):
            ret.append(self.__to_id(num, alphabet[1:]))

            if i >= len(numbers) - 1:
                continue

            ret.append(alphabet[0])
            alphabet = self.__shuffle(alphabet)

        id_ = "".join(ret)

        if self.__min_length > len(id_):
            id_ += alphabet[0]

            while self.__min_length - len(id_) > 0:
                alphabet = self.__shuffle(alphabet)
                id_ += alphabet[: min(self.__min_length - len(id_), len(alphabet))]

        if len(id_) >= 3 and self.__is_blocked_id(id_):
            id_ = self.__encode_numbers(numbers, increment + 1)

        return id_

    def decode(self, id_: str) -> List[int]:
        ret: List[int] = []

        if not id_:
            return ret

        alphabet_chars = list(self.__alphabet)
        if any(c not in alphabet_chars for c in id_):
            return ret

        prefix = id_[0]
        offset = self.__alphabet.index(prefix)
        alphabet = self.__alphabet[offset:] + self.__alphabet[:offset]
        alphabet = alphabet[::-1]
        id_ = id_[1:]

        while id_:
            separator = alphabet[0]
            chunks = id_.split(separator)
            if not chunks[0]:
                return ret

            ret.append(self.__to_number(chunks[0], alphabet[1:]))
            if len(chunks) > 1:
                alphabet = self.__shuffle(alphabet)

            id_ = separator.join(chunks[1:])

        return ret

    def __shuffle(self, alphabet: str) -> str:
        chars = list(alphabet)

        i = 0
        j = len(chars) - 1
        while j > 0:
            r = (i * j + ord(chars[i]) + ord(chars[j])) % len(chars)
            chars[i], chars[r] = chars[r], chars[i]
            i += 1
            j -= 1

        return "".join(chars)

    def __to_id(self, num: int, alphabet: str) -> str:
        id_chars: List[str] = []
        result = num
        alphabet_length = len(alphabet)

        while True:
            id_chars.insert(0, alphabet[result % alphabet_length])
            result = result // alphabet_length
            if not result:
                break

        return "".join(id_chars)

    def __to_number(self, id_: str, alphabet: str) -> int:
        chars = list(alphabet)
        return sum(chars.index(c) * (len(chars) ** i) for i, c in enumerate(id_[::-1]))

    def __is_blocked_id(self, id_: str) -> bool:
        id_ = id_.lower()

        if len(id_) == 3:
            return id_ in self.__blocklist_exact_match

        if (
            id_.startswith(self.__blocklist_match_at_ends)
            or id_.endswith(self.__blocklist_match_at_ends)
        ):
            return True

        for word in self.__blocklist_match_anywhere:
            if word in id_:
                return True

        return False



import sqids
import timeit

number = 100_000

print(f"Iterations: {number:,d}")

print(
    "{0:<20s} {1:7.3f}".format(
        "Instantiate:",
        timeit.timeit(
            stmt="sqids.Sqids()",
            globals={"sqids": sqids},
            number=number,
        )
    )
)

print(
    "{0:<20s} {1:7.3f}".format(
        "Encode [0]:",  # [0] -> 'bM'
        timeit.timeit(
            stmt="squid.encode([0])",
            globals={"squid": sqids.Sqids()},
            number=number,
        )
    )
)

print(
    "{0:<20s} {1:7.3f}".format(
        "Encode [0, 1, 2]:",  # [0, 1, 2] -> 'rSCtlB'
        timeit.timeit(
            stmt="squid.encode([0, 1, 2])",
            globals={"squid": sqids.Sqids()},
            number=number,
        )
    )
)

print(
    "{0:<20s} {1:7.3f}".format(
        "Decode 'bM':",  # 'bM' -> [0]
        timeit.timeit(
            stmt="squid.decode('bM')",
            globals={"squid": sqids.Sqids()},
            number=number,
        )
    )
)

print(
    "{0:<20s} {1:7.3f}".format(
        "Decode 'rSCtlB':",  # 'rSCtlB' -> [0, 1, 2]
        timeit.timeit(
            stmt="squid.decode('rSCtlB')",
            globals={"squid": sqids.Sqids()},
            number=number,
        )
    ),
)



import pathlib
import sys
from typing import Set, Tuple


repo_root = pathlib.Path(__file__).parent.parent
this_file = pathlib.Path(__file__).relative_to(repo_root)
constants_path = repo_root / "sqids/constants.py"
import sqids.constants  # noqa


DIGITS = set("0123456789")


def filter_blocklist() -> Tuple[Set[str], Set[str], Set[str]]:
    """Pre-filter the blocklist and update the constants file."""

    exact_match = set()
    match_at_ends = set()
    match_anywhere = set()

    for word in sqids.constants.DEFAULT_BLOCKLIST:
        if len(word) == 3:
            exact_match.add(word)
        elif set(word) & DIGITS:
            match_at_ends.add(word)
        else:
            match_anywhere.add(word)

    return exact_match, match_at_ends, match_anywhere


def generate_new_constants_file(
    exact_match: Set[str],
    match_at_ends: Set[str],
    match_anywhere: Set[str],
) -> str:
    """Generate the text of a new constants file."""

    lines = [
        f'DEFAULT_ALPHABET = "{sqids.constants.DEFAULT_ALPHABET}"',
        f"DEFAULT_MIN_LENGTH = {sqids.constants.DEFAULT_MIN_LENGTH}",
        "",
        "# =======",
        "#  NOTE",
        "# =======",
        "#",
        f"# When updating the blocklist, run {this_file} to pre-filter constants.",
        "# This is critical for performance.",
        "#",
        "",
        "DEFAULT_BLOCKLIST = [",
    ]
    # Output a sorted blocklist.
    for word in sorted(sqids.constants.DEFAULT_BLOCKLIST):
        lines.append(f'    "{word}",')
    lines.append("]")

    # Output exact-match blocklist words.
    lines.append("")
    lines.append("_exact_match = {")
    for word in sorted(exact_match):
        lines.append(f'    "{word}",')
    lines.append("}")

    # Output match-at-ends blocklist words.
    lines.append("")
    lines.append("_match_at_ends = (")
    for word in sorted(match_at_ends):
        lines.append(f'    "{word}",')
    lines.append(")")

    # Output match-anywhere blocklist words.
    lines.append("")
    lines.append("_match_anywhere = {")
    for word in sorted(match_anywhere):
        lines.append(f'    "{word}",')
    lines.append("}")

    return "\n".join(lines).rstrip() + "\n"  # Include a trailing newline.


def main() -> int:
    text = constants_path.read_text()

    exact_match, match_at_ends, match_anywhere = filter_blocklist()
    new_text = generate_new_constants_file(exact_match, match_at_ends, match_anywhere)

    if text == new_text:
        print("No changes necessary")
        return 0

    print(f"Updating {constants_path.relative_to(repo_root)}")
    constants_path.write_text(new_text, newline="\n", encoding="utf-8")
    return 1


if __name__ == "__main__":
    sys.exit(main())



import pathlib
import sys

import pytest
from sqids import Sqids


def test_default_blocklist():
    sqids = Sqids()

    assert sqids.decode("aho1e") == [4572721]
    assert sqids.encode([4572721]) == "JExTR"


def test_empty_blocklist():
    sqids = Sqids(blocklist=set())

    assert sqids.decode("aho1e") == [4572721]
    assert sqids.encode([4572721]) == "aho1e"


def test_custom_blocklist():
    sqids = Sqids(blocklist={"ArUO"})

    assert sqids.decode("aho1e") == [4572721]
    assert sqids.encode([4572721]) == "aho1e"

    assert sqids.decode("ArUO") == [100000]
    assert sqids.encode([100000]) == "QyG4"
    assert sqids.decode("QyG4") == [100000]


def test_blocklist():
    sqids = Sqids(
        blocklist={
            "JSwXFaosAN",  # normal result of 1st encoding, block that word on purpose
            "OCjV9JK64o",  # result of 2nd encoding
            "rBHf",  # result of 3rd encoding is `4rBHfOiqd3`, let's block a substring
            "79SM",  # result of 4th encoding is `dyhgw479SM`, let's block the postfix
            "7tE6",  # result of 4th encoding is `7tE6jdAHLe`, let's block the prefix
        }
    )

    assert sqids.encode([1000000, 2000000]) == "1aYeB7bRUt"
    assert sqids.decode("1aYeB7bRUt") == [1000000, 2000000]


def test_decoding_blocklist_words():
    sqids = Sqids(blocklist={"86Rf07", "se8ojk", "ARsz1p", "Q8AI49", "5sQRZO"})

    assert sqids.decode("86Rf07") == [1, 2, 3]
    assert sqids.decode("se8ojk") == [1, 2, 3]
    assert sqids.decode("ARsz1p") == [1, 2, 3]
    assert sqids.decode("Q8AI49") == [1, 2, 3]
    assert sqids.decode("5sQRZO") == [1, 2, 3]


def test_match_against_short_blocklist_word():
    sqids = Sqids(blocklist={"pnd"})

    assert sqids.decode(sqids.encode([1000])) == [1000]


def test_blocklist_filtering_in_constructor():
    # lowercase blocklist in only-uppercase alphabet
    sqids = Sqids(alphabet="ABCDEFGHIJKLMNOPQRSTUVWXYZ", blocklist={"sxnzkl"})

    id = sqids.encode([1, 2, 3])
    numbers = sqids.decode(id)

    # without blocklist, would've been "SXNZKL"
    assert id == "IBSHOZ"
    assert numbers == [1, 2, 3]


@pytest.mark.parametrize("word", ("ab!", "abc!", "xyz"))
def test_alphabet_is_not_superset_of_blocklist_word_characters(word):
    """Verify that a non-subset blocklist word is ignored."""

    sqids = Sqids(alphabet="abc", blocklist=[word])
    assert sqids._Sqids__blocklist_exact_match == set()
    assert sqids._Sqids__blocklist_match_at_ends == tuple()
    assert sqids._Sqids__blocklist_match_anywhere == set()


def test_max_encoding_attempts():
    alphabet = "abc"
    min_length = 3
    blocklist = {"cab", "abc", "bca"}

    sqids = Sqids(alphabet, min_length, blocklist)

    assert min_length == len(alphabet)
    assert min_length == len(blocklist)

    with pytest.raises(Exception):
        sqids.encode([0])


def test_small_words_are_ignored():
    """Blocklist words shorter than 3 characters must be ignored."""

    id_ = Sqids().encode([0])
    assert id_ == "bM"
    id_ = Sqids(blocklist=[id_]).encode([0])
    assert id_ == "bM"


def test_constants_file_is_pristine():
    """Verify the constants file is pristine."""

    repo_root = pathlib.Path(__file__).parent.parent
    sys.path.append(str(repo_root / "assets"))
    import filter_blocklist

    sets = filter_blocklist.filter_blocklist()
    new_text = filter_blocklist.generate_new_constants_file(*sets)
    error_message = "You must run assets/filter_blocklist.py!"
    assert filter_blocklist.constants_path.read_text() == new_text, error_message



DEFAULT_ALPHABET = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
DEFAULT_MIN_LENGTH = 0

# =======
#  NOTE
# =======
#
# When updating the blocklist, run assets/filter_blocklist.py to pre-filter constants.
# This is critical for performance.
#

DEFAULT_BLOCKLIST = [
    "0rgasm",
    "1d10t",
    "1d1ot",
    "1di0t",
    "1diot",
    "1eccacu10",
    "1eccacu1o",
    "1eccacul0",
    "1eccaculo",
    "1mbec11e",
    "1mbec1le",
    "1mbeci1e",
    "1mbecile",
    "a11upat0",
    "a11upato",
    "a1lupat0",
    "a1lupato",
    "aand",
    "ah01e",
    "ah0le",
    "aho1e",
    "ahole",
    "al1upat0",
    "al1upato",
    "allupat0",
    "allupato",
    "ana1",
    "ana1e",
    "anal",
    "anale",
    "anus",
    "arrapat0",
    "arrapato",
    "arsch",
    "arse",
    "ass",
    "b00b",
    "b00be",
    "b01ata",
    "b0ceta",
    "b0iata",
    "b0ob",
    "b0obe",
    "b0sta",
    "b1tch",
    "b1te",
    "b1tte",
    "ba1atkar",
    "balatkar",
    "bastard0",
    "bastardo",
    "batt0na",
    "battona",
    "bitch",
    "bite",
    "bitte",
    "bo0b",
    "bo0be",
    "bo1ata",
    "boceta",
    "boiata",
    "boob",
    "boobe",
    "bosta",
    "bran1age",
    "bran1er",
    "bran1ette",
    "bran1eur",
    "bran1euse",
    "branlage",
    "branler",
    "branlette",
    "branleur",
    "branleuse",
    "c0ck",
    "c0g110ne",
    "c0g11one",
    "c0g1i0ne",
    "c0g1ione",
    "c0gl10ne",
    "c0gl1one",
    "c0gli0ne",
    "c0glione",
    "c0na",
    "c0nnard",
    "c0nnasse",
    "c0nne",
    "c0u111es",
    "c0u11les",
    "c0u1l1es",
    "c0u1lles",
    "c0ui11es",
    "c0ui1les",
    "c0uil1es",
    "c0uilles",
    "c11t",
    "c11t0",
    "c11to",
    "c1it",
    "c1it0",
    "c1ito",
    "cabr0n",
    "cabra0",
    "cabrao",
    "cabron",
    "caca",
    "cacca",
    "cacete",
    "cagante",
    "cagar",
    "cagare",
    "cagna",
    "cara1h0",
    "cara1ho",
    "caracu10",
    "caracu1o",
    "caracul0",
    "caraculo",
    "caralh0",
    "caralho",
    "cazz0",
    "cazz1mma",
    "cazzata",
    "cazzimma",
    "cazzo",
    "ch00t1a",
    "ch00t1ya",
    "ch00tia",
    "ch00tiya",
    "ch0d",
    "ch0ot1a",
    "ch0ot1ya",
    "ch0otia",
    "ch0otiya",
    "ch1asse",
    "ch1avata",
    "ch1er",
    "ch1ng0",
    "ch1ngadaz0s",
    "ch1ngadazos",
    "ch1ngader1ta",
    "ch1ngaderita",
    "ch1ngar",
    "ch1ngo",
    "ch1ngues",
    "ch1nk",
    "chatte",
    "chiasse",
    "chiavata",
    "chier",
    "ching0",
    "chingadaz0s",
    "chingadazos",
    "chingader1ta",
    "chingaderita",
    "chingar",
    "chingo",
    "chingues",
    "chink",
    "cho0t1a",
    "cho0t1ya",
    "cho0tia",
    "cho0tiya",
    "chod",
    "choot1a",
    "choot1ya",
    "chootia",
    "chootiya",
    "cl1t",
    "cl1t0",
    "cl1to",
    "clit",
    "clit0",
    "clito",
    "cock",
    "cog110ne",
    "cog11one",
    "cog1i0ne",
    "cog1ione",
    "cogl10ne",
    "cogl1one",
    "cogli0ne",
    "coglione",
    "cona",
    "connard",
    "connasse",
    "conne",
    "cou111es",
    "cou11les",
    "cou1l1es",
    "cou1lles",
    "coui11es",
    "coui1les",
    "couil1es",
    "couilles",
    "cracker",
    "crap",
    "cu10",
    "cu1att0ne",
    "cu1attone",
    "cu1er0",
    "cu1ero",
    "cu1o",
    "cul0",
    "culatt0ne",
    "culattone",
    "culer0",
    "culero",
    "culo",
    "cum",
    "cunt",
    "d11d0",
    "d11do",
    "d1ck",
    "d1ld0",
    "d1ldo",
    "damn",
    "de1ch",
    "deich",
    "depp",
    "di1d0",
    "di1do",
    "dick",
    "dild0",
    "dildo",
    "dyke",
    "encu1e",
    "encule",
    "enema",
    "enf01re",
    "enf0ire",
    "enfo1re",
    "enfoire",
    "estup1d0",
    "estup1do",
    "estupid0",
    "estupido",
    "etr0n",
    "etron",
    "f0da",
    "f0der",
    "f0ttere",
    "f0tters1",
    "f0ttersi",
    "f0tze",
    "f0utre",
    "f1ca",
    "f1cker",
    "f1ga",
    "fag",
    "fica",
    "ficker",
    "figa",
    "foda",
    "foder",
    "fottere",
    "fotters1",
    "fottersi",
    "fotze",
    "foutre",
    "fr0c10",
    "fr0c1o",
    "fr0ci0",
    "fr0cio",
    "fr0sc10",
    "fr0sc1o",
    "fr0sci0",
    "fr0scio",
    "froc10",
    "froc1o",
    "froci0",
    "frocio",
    "frosc10",
    "frosc1o",
    "frosci0",
    "froscio",
    "fuck",
    "g00",
    "g0o",
    "g0u1ne",
    "g0uine",
    "gandu",
    "go0",
    "goo",
    "gou1ne",
    "gouine",
    "gr0gnasse",
    "grognasse",
    "haram1",
    "harami",
    "haramzade",
    "hund1n",
    "hundin",
    "id10t",
    "id1ot",
    "idi0t",
    "idiot",
    "imbec11e",
    "imbec1le",
    "imbeci1e",
    "imbecile",
    "j1zz",
    "jerk",
    "jizz",
    "k1ke",
    "kam1ne",
    "kamine",
    "kike",
    "leccacu10",
    "leccacu1o",
    "leccacul0",
    "leccaculo",
    "m1erda",
    "m1gn0tta",
    "m1gnotta",
    "m1nch1a",
    "m1nchia",
    "m1st",
    "mam0n",
    "mamahuev0",
    "mamahuevo",
    "mamon",
    "masturbat10n",
    "masturbat1on",
    "masturbate",
    "masturbati0n",
    "masturbation",
    "merd0s0",
    "merd0so",
    "merda",
    "merde",
    "merdos0",
    "merdoso",
    "mierda",
    "mign0tta",
    "mignotta",
    "minch1a",
    "minchia",
    "mist",
    "musch1",
    "muschi",
    "n1gger",
    "neger",
    "negr0",
    "negre",
    "negro",
    "nerch1a",
    "nerchia",
    "nigger",
    "orgasm",
    "p00p",
    "p011a",
    "p01la",
    "p0l1a",
    "p0lla",
    "p0mp1n0",
    "p0mp1no",
    "p0mpin0",
    "p0mpino",
    "p0op",
    "p0rca",
    "p0rn",
    "p0rra",
    "p0uff1asse",
    "p0uffiasse",
    "p1p1",
    "p1pi",
    "p1r1a",
    "p1rla",
    "p1sc10",
    "p1sc1o",
    "p1sci0",
    "p1scio",
    "p1sser",
    "pa11e",
    "pa1le",
    "pal1e",
    "palle",
    "pane1e1r0",
    "pane1e1ro",
    "pane1eir0",
    "pane1eiro",
    "panele1r0",
    "panele1ro",
    "paneleir0",
    "paneleiro",
    "patakha",
    "pec0r1na",
    "pec0rina",
    "pecor1na",
    "pecorina",
    "pen1s",
    "pendej0",
    "pendejo",
    "penis",
    "pip1",
    "pipi",
    "pir1a",
    "pirla",
    "pisc10",
    "pisc1o",
    "pisci0",
    "piscio",
    "pisser",
    "po0p",
    "po11a",
    "po1la",
    "pol1a",
    "polla",
    "pomp1n0",
    "pomp1no",
    "pompin0",
    "pompino",
    "poop",
    "porca",
    "porn",
    "porra",
    "pouff1asse",
    "pouffiasse",
    "pr1ck",
    "prick",
    "pussy",
    "put1za",
    "puta",
    "puta1n",
    "putain",
    "pute",
    "putiza",
    "puttana",
    "queca",
    "r0mp1ba11e",
    "r0mp1ba1le",
    "r0mp1bal1e",
    "r0mp1balle",
    "r0mpiba11e",
    "r0mpiba1le",
    "r0mpibal1e",
    "r0mpiballe",
    "rand1",
    "randi",
    "rape",
    "recch10ne",
    "recch1one",
    "recchi0ne",
    "recchione",
    "retard",
    "romp1ba11e",
    "romp1ba1le",
    "romp1bal1e",
    "romp1balle",
    "rompiba11e",
    "rompiba1le",
    "rompibal1e",
    "rompiballe",
    "ruff1an0",
    "ruff1ano",
    "ruffian0",
    "ruffiano",
    "s1ut",
    "sa10pe",
    "sa1aud",
    "sa1ope",
    "sacanagem",
    "sal0pe",
    "salaud",
    "salope",
    "saugnapf",
    "sb0rr0ne",
    "sb0rra",
    "sb0rrone",
    "sbattere",
    "sbatters1",
    "sbattersi",
    "sborr0ne",
    "sborra",
    "sborrone",
    "sc0pare",
    "sc0pata",
    "sch1ampe",
    "sche1se",
    "sche1sse",
    "scheise",
    "scheisse",
    "schlampe",
    "schwachs1nn1g",
    "schwachs1nnig",
    "schwachsinn1g",
    "schwachsinnig",
    "schwanz",
    "scopare",
    "scopata",
    "sexy",
    "sh1t",
    "shit",
    "slut",
    "sp0mp1nare",
    "sp0mpinare",
    "spomp1nare",
    "spompinare",
    "str0nz0",
    "str0nza",
    "str0nzo",
    "stronz0",
    "stronza",
    "stronzo",
    "stup1d",
    "stupid",
    "succh1am1",
    "succh1ami",
    "succhiam1",
    "succhiami",
    "sucker",
    "t0pa",
    "tapette",
    "test1c1e",
    "test1cle",
    "testic1e",
    "testicle",
    "tette",
    "topa",
    "tr01a",
    "tr0ia",
    "tr0mbare",
    "tr1ng1er",
    "tr1ngler",
    "tring1er",
    "tringler",
    "tro1a",
    "troia",
    "trombare",
    "turd",
    "twat",
    "vaffancu10",
    "vaffancu1o",
    "vaffancul0",
    "vaffanculo",
    "vag1na",
    "vagina",
    "verdammt",
    "verga",
    "w1chsen",
    "wank",
    "wichsen",
    "x0ch0ta",
    "x0chota",
    "xana",
    "xoch0ta",
    "xochota",
    "z0cc01a",
    "z0cc0la",
    "z0cco1a",
    "z0ccola",
    "z1z1",
    "z1zi",
    "ziz1",
    "zizi",
    "zocc01a",
    "zocc0la",
    "zocco1a",
    "zoccola",
]

_exact_match = {
    "ass",
    "cum",
    "fag",
    "g00",
    "g0o",
    "go0",
    "goo",
}

_match_at_ends = (
    "0rgasm",
    "1d10t",
    "1d1ot",
    "1di0t",
    "1diot",
    "1eccacu10",
    "1eccacu1o",
    "1eccacul0",
    "1eccaculo",
    "1mbec11e",
    "1mbec1le",
    "1mbeci1e",
    "1mbecile",
    "a11upat0",
    "a11upato",
    "a1lupat0",
    "a1lupato",
    "ah01e",
    "ah0le",
    "aho1e",
    "al1upat0",
    "al1upato",
    "allupat0",
    "ana1",
    "ana1e",
    "arrapat0",
    "b00b",
    "b00be",
    "b01ata",
    "b0ceta",
    "b0iata",
    "b0ob",
    "b0obe",
    "b0sta",
    "b1tch",
    "b1te",
    "b1tte",
    "ba1atkar",
    "bastard0",
    "batt0na",
    "bo0b",
    "bo0be",
    "bo1ata",
    "bran1age",
    "bran1er",
    "bran1ette",
    "bran1eur",
    "bran1euse",
    "c0ck",
    "c0g110ne",
    "c0g11one",
    "c0g1i0ne",
    "c0g1ione",
    "c0gl10ne",
    "c0gl1one",
    "c0gli0ne",
    "c0glione",
    "c0na",
    "c0nnard",
    "c0nnasse",
    "c0nne",
    "c0u111es",
    "c0u11les",
    "c0u1l1es",
    "c0u1lles",
    "c0ui11es",
    "c0ui1les",
    "c0uil1es",
    "c0uilles",
    "c11t",
    "c11t0",
    "c11to",
    "c1it",
    "c1it0",
    "c1ito",
    "cabr0n",
    "cabra0",
    "cara1h0",
    "cara1ho",
    "caracu10",
    "caracu1o",
    "caracul0",
    "caralh0",
    "cazz0",
    "cazz1mma",
    "ch00t1a",
    "ch00t1ya",
    "ch00tia",
    "ch00tiya",
    "ch0d",
    "ch0ot1a",
    "ch0ot1ya",
    "ch0otia",
    "ch0otiya",
    "ch1asse",
    "ch1avata",
    "ch1er",
    "ch1ng0",
    "ch1ngadaz0s",
    "ch1ngadazos",
    "ch1ngader1ta",
    "ch1ngaderita",
    "ch1ngar",
    "ch1ngo",
    "ch1ngues",
    "ch1nk",
    "ching0",
    "chingadaz0s",
    "chingader1ta",
    "cho0t1a",
    "cho0t1ya",
    "cho0tia",
    "cho0tiya",
    "choot1a",
    "choot1ya",
    "cl1t",
    "cl1t0",
    "cl1to",
    "clit0",
    "cog110ne",
    "cog11one",
    "cog1i0ne",
    "cog1ione",
    "cogl10ne",
    "cogl1one",
    "cogli0ne",
    "cou111es",
    "cou11les",
    "cou1l1es",
    "cou1lles",
    "coui11es",
    "coui1les",
    "couil1es",
    "cu10",
    "cu1att0ne",
    "cu1attone",
    "cu1er0",
    "cu1ero",
    "cu1o",
    "cul0",
    "culatt0ne",
    "culer0",
    "d11d0",
    "d11do",
    "d1ck",
    "d1ld0",
    "d1ldo",
    "de1ch",
    "di1d0",
    "di1do",
    "dild0",
    "encu1e",
    "enf01re",
    "enf0ire",
    "enfo1re",
    "estup1d0",
    "estup1do",
    "estupid0",
    "etr0n",
    "f0da",
    "f0der",
    "f0ttere",
    "f0tters1",
    "f0ttersi",
    "f0tze",
    "f0utre",
    "f1ca",
    "f1cker",
    "f1ga",
    "fotters1",
    "fr0c10",
    "fr0c1o",
    "fr0ci0",
    "fr0cio",
    "fr0sc10",
    "fr0sc1o",
    "fr0sci0",
    "fr0scio",
    "froc10",
    "froc1o",
    "froci0",
    "frosc10",
    "frosc1o",
    "frosci0",
    "g0u1ne",
    "g0uine",
    "gou1ne",
    "gr0gnasse",
    "haram1",
    "hund1n",
    "id10t",
    "id1ot",
    "idi0t",
    "imbec11e",
    "imbec1le",
    "imbeci1e",
    "j1zz",
    "k1ke",
    "kam1ne",
    "leccacu10",
    "leccacu1o",
    "leccacul0",
    "m1erda",
    "m1gn0tta",
    "m1gnotta",
    "m1nch1a",
    "m1nchia",
    "m1st",
    "mam0n",
    "mamahuev0",
    "masturbat10n",
    "masturbat1on",
    "masturbati0n",
    "merd0s0",
    "merd0so",
    "merdos0",
    "mign0tta",
    "minch1a",
    "musch1",
    "n1gger",
    "negr0",
    "nerch1a",
    "p00p",
    "p011a",
    "p01la",
    "p0l1a",
    "p0lla",
    "p0mp1n0",
    "p0mp1no",
    "p0mpin0",
    "p0mpino",
    "p0op",
    "p0rca",
    "p0rn",
    "p0rra",
    "p0uff1asse",
    "p0uffiasse",
    "p1p1",
    "p1pi",
    "p1r1a",
    "p1rla",
    "p1sc10",
    "p1sc1o",
    "p1sci0",
    "p1scio",
    "p1sser",
    "pa11e",
    "pa1le",
    "pal1e",
    "pane1e1r0",
    "pane1e1ro",
    "pane1eir0",
    "pane1eiro",
    "panele1r0",
    "panele1ro",
    "paneleir0",
    "pec0r1na",
    "pec0rina",
    "pecor1na",
    "pen1s",
    "pendej0",
    "pip1",
    "pir1a",
    "pisc10",
    "pisc1o",
    "pisci0",
    "po0p",
    "po11a",
    "po1la",
    "pol1a",
    "pomp1n0",
    "pomp1no",
    "pompin0",
    "pouff1asse",
    "pr1ck",
    "put1za",
    "puta1n",
    "r0mp1ba11e",
    "r0mp1ba1le",
    "r0mp1bal1e",
    "r0mp1balle",
    "r0mpiba11e",
    "r0mpiba1le",
    "r0mpibal1e",
    "r0mpiballe",
    "rand1",
    "recch10ne",
    "recch1one",
    "recchi0ne",
    "romp1ba11e",
    "romp1ba1le",
    "romp1bal1e",
    "romp1balle",
    "rompiba11e",
    "rompiba1le",
    "rompibal1e",
    "ruff1an0",
    "ruff1ano",
    "ruffian0",
    "s1ut",
    "sa10pe",
    "sa1aud",
    "sa1ope",
    "sal0pe",
    "sb0rr0ne",
    "sb0rra",
    "sb0rrone",
    "sbatters1",
    "sborr0ne",
    "sc0pare",
    "sc0pata",
    "sch1ampe",
    "sche1se",
    "sche1sse",
    "schwachs1nn1g",
    "schwachs1nnig",
    "schwachsinn1g",
    "sh1t",
    "sp0mp1nare",
    "sp0mpinare",
    "spomp1nare",
    "str0nz0",
    "str0nza",
    "str0nzo",
    "stronz0",
    "stup1d",
    "succh1am1",
    "succh1ami",
    "succhiam1",
    "t0pa",
    "test1c1e",
    "test1cle",
    "testic1e",
    "tr01a",
    "tr0ia",
    "tr0mbare",
    "tr1ng1er",
    "tr1ngler",
    "tring1er",
    "tro1a",
    "vaffancu10",
    "vaffancu1o",
    "vaffancul0",
    "vag1na",
    "w1chsen",
    "x0ch0ta",
    "x0chota",
    "xoch0ta",
    "z0cc01a",
    "z0cc0la",
    "z0cco1a",
    "z0ccola",
    "z1z1",
    "z1zi",
    "ziz1",
    "zocc01a",
    "zocc0la",
    "zocco1a",
)

_match_anywhere = {
    "aand",
    "ahole",
    "allupato",
    "anal",
    "anale",
    "anus",
    "arrapato",
    "arsch",
    "arse",
    "balatkar",
    "bastardo",
    "battona",
    "bitch",
    "bite",
    "bitte",
    "boceta",
    "boiata",
    "boob",
    "boobe",
    "bosta",
    "branlage",
    "branler",
    "branlette",
    "branleur",
    "branleuse",
    "cabrao",
    "cabron",
    "caca",
    "cacca",
    "cacete",
    "cagante",
    "cagar",
    "cagare",
    "cagna",
    "caraculo",
    "caralho",
    "cazzata",
    "cazzimma",
    "cazzo",
    "chatte",
    "chiasse",
    "chiavata",
    "chier",
    "chingadazos",
    "chingaderita",
    "chingar",
    "chingo",
    "chingues",
    "chink",
    "chod",
    "chootia",
    "chootiya",
    "clit",
    "clito",
    "cock",
    "coglione",
    "cona",
    "connard",
    "connasse",
    "conne",
    "couilles",
    "cracker",
    "crap",
    "culattone",
    "culero",
    "culo",
    "cunt",
    "damn",
    "deich",
    "depp",
    "dick",
    "dildo",
    "dyke",
    "encule",
    "enema",
    "enfoire",
    "estupido",
    "etron",
    "fica",
    "ficker",
    "figa",
    "foda",
    "foder",
    "fottere",
    "fottersi",
    "fotze",
    "foutre",
    "frocio",
    "froscio",
    "fuck",
    "gandu",
    "gouine",
    "grognasse",
    "harami",
    "haramzade",
    "hundin",
    "idiot",
    "imbecile",
    "jerk",
    "jizz",
    "kamine",
    "kike",
    "leccaculo",
    "mamahuevo",
    "mamon",
    "masturbate",
    "masturbation",
    "merda",
    "merde",
    "merdoso",
    "mierda",
    "mignotta",
    "minchia",
    "mist",
    "muschi",
    "neger",
    "negre",
    "negro",
    "nerchia",
    "nigger",
    "orgasm",
    "palle",
    "paneleiro",
    "patakha",
    "pecorina",
    "pendejo",
    "penis",
    "pipi",
    "pirla",
    "piscio",
    "pisser",
    "polla",
    "pompino",
    "poop",
    "porca",
    "porn",
    "porra",
    "pouffiasse",
    "prick",
    "pussy",
    "puta",
    "putain",
    "pute",
    "putiza",
    "puttana",
    "queca",
    "randi",
    "rape",
    "recchione",
    "retard",
    "rompiballe",
    "ruffiano",
    "sacanagem",
    "salaud",
    "salope",
    "saugnapf",
    "sbattere",
    "sbattersi",
    "sborra",
    "sborrone",
    "scheise",
    "scheisse",
    "schlampe",
    "schwachsinnig",
    "schwanz",
    "scopare",
    "scopata",
    "sexy",
    "shit",
    "slut",
    "spompinare",
    "stronza",
    "stronzo",
    "stupid",
    "succhiami",
    "sucker",
    "tapette",
    "testicle",
    "tette",
    "topa",
    "tringler",
    "troia",
    "trombare",
    "turd",
    "twat",
    "vaffanculo",
    "vagina",
    "verdammt",
    "verga",
    "wank",
    "wichsen",
    "xana",
    "xochota",
    "zizi",
    "zoccola",
}



import pytest
from sqids import Sqids


def test_simple():
    sqids = Sqids(alphabet="0123456789abcdef")

    numbers = [1, 2, 3]
    id = "489158"

    assert sqids.encode(numbers) == id
    assert sqids.decode(id) == numbers


def test_short_alphabet():
    sqids = Sqids(alphabet="abc")

    numbers = [1, 2, 3]
    assert sqids.decode(sqids.encode(numbers)) == numbers


def test_long_alphabet():
    sqids = Sqids(
        alphabet="abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()-_+|{}[];:'\"/?.>,<`~"
    )

    numbers = [1, 2, 3]
    assert sqids.decode(sqids.encode(numbers)) == numbers


def test_multibyte_alphabet():
    with pytest.raises(Exception):
        Sqids(alphabet="ë1092")


def test_repeating_alphabet_characters():
    with pytest.raises(Exception):
        Sqids(alphabet="aabcdefg")


def test_too_short_alphabet():
    with pytest.raises(Exception):
        Sqids(alphabet="ab")



import pytest
import sys
from sqids import Sqids
from sqids.constants import DEFAULT_ALPHABET


def test_simple():
    sqids = Sqids(min_length=len(DEFAULT_ALPHABET))

    numbers = [1, 2, 3]
    id_str = "86Rf07xd4zBmiJXQG6otHEbew02c3PWsUOLZxADhCpKj7aVFv9I8RquYrNlSTM"

    assert sqids.encode(numbers) == id_str
    assert sqids.decode(id_str) == numbers


@pytest.mark.parametrize(
    "min_length, expected_id",
    (
        (6, "86Rf07"),
        (7, "86Rf07x"),
        (8, "86Rf07xd"),
        (9, "86Rf07xd4"),
        (10, "86Rf07xd4z"),
        (11, "86Rf07xd4zB"),
        (12, "86Rf07xd4zBm"),
        (13, "86Rf07xd4zBmi"),
        (
            len(DEFAULT_ALPHABET) + 0,
            "86Rf07xd4zBmiJXQG6otHEbew02c3PWsUOLZxADhCpKj7aVFv9I8RquYrNlSTM",
        ),
        (
            len(DEFAULT_ALPHABET) + 1,
            "86Rf07xd4zBmiJXQG6otHEbew02c3PWsUOLZxADhCpKj7aVFv9I8RquYrNlSTMy",
        ),
        (
            len(DEFAULT_ALPHABET) + 2,
            "86Rf07xd4zBmiJXQG6otHEbew02c3PWsUOLZxADhCpKj7aVFv9I8RquYrNlSTMyf",
        ),
        (
            len(DEFAULT_ALPHABET) + 3,
            "86Rf07xd4zBmiJXQG6otHEbew02c3PWsUOLZxADhCpKj7aVFv9I8RquYrNlSTMyf1",
        ),
    )
)
def test_incremental(min_length, expected_id):
    numbers = [1, 2, 3]
    sqids = Sqids(min_length=min_length)

    assert sqids.encode(numbers) == expected_id
    assert len(sqids.encode(numbers)) == min_length
    assert sqids.decode(expected_id) == numbers


@pytest.mark.parametrize(
    "id_str, numbers",
    (
        ("SvIzsqYMyQwI3GWgJAe17URxX8V924Co0DaTZLtFjHriEn5bPhcSkfmvOslpBu", [0, 0]),
        ("n3qafPOLKdfHpuNw3M61r95svbeJGk7aAEgYn4WlSjXURmF8IDqZBy0CT2VxQc", [0, 1]),
        ("tryFJbWcFMiYPg8sASm51uIV93GXTnvRzyfLleh06CpodJD42B7OraKtkQNxUZ", [0, 2]),
        ("eg6ql0A3XmvPoCzMlB6DraNGcWSIy5VR8iYup2Qk4tjZFKe1hbwfgHdUTsnLqE", [0, 3]),
        ("rSCFlp0rB2inEljaRdxKt7FkIbODSf8wYgTsZM1HL9JzN35cyoqueUvVWCm4hX", [0, 4]),
        ("sR8xjC8WQkOwo74PnglH1YFdTI0eaf56RGVSitzbjuZ3shNUXBrqLxEJyAmKv2", [0, 5]),
        ("uY2MYFqCLpgx5XQcjdtZK286AwWV7IBGEfuS9yTmbJvkzoUPeYRHr4iDs3naN0", [0, 6]),
        ("74dID7X28VLQhBlnGmjZrec5wTA1fqpWtK4YkaoEIM9SRNiC3gUJH0OFvsPDdy", [0, 7]),
        ("30WXpesPhgKiEI5RHTY7xbB1GnytJvXOl2p0AcUjdF6waZDo9Qk8VLzMuWrqCS", [0, 8]),
        ("moxr3HqLAK0GsTND6jowfZz3SUx7cQ8aC54Pl1RbIvFXmEJuBMYVeW9yrdOtin", [0, 9]),
    )
)
def test_incremental_numbers(id_str, numbers):
    sqids = Sqids(min_length=len(DEFAULT_ALPHABET))

    assert sqids.encode(numbers) == id_str
    assert sqids.decode(id_str) == numbers


@pytest.mark.parametrize("min_length", (0, 1, 5, 10, len(DEFAULT_ALPHABET)))
@pytest.mark.parametrize(
    "numbers",
    (
        [0],
        [0, 0, 0, 0, 0],
        [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        [100, 200, 300],
        [1000, 2000, 3000],
        [1000000],
        [sys.maxsize],
    ),
)
def test_min_lengths(min_length, numbers):
    sqids = Sqids(min_length=min_length)

    id_str = sqids.encode(numbers)
    assert len(id_str) >= min_length
    assert sqids.decode(id_str) == numbers


def test_out_of_range_invalid_min_length():
    with pytest.raises(ValueError):
        Sqids(min_length=-1)

    with pytest.raises(ValueError):
        Sqids(min_length=256)


@pytest.mark.parametrize("min_length", ("bogus", {}, True))
def test_min_length_type(min_length):
    """Verify that non-integer min_length values are rejected.

    `True` is a unique case; Python bools are subclasses of int.
    """

    with pytest.raises(TypeError, match="must be an int"):
        Sqids(min_length=min_length)
